@@ -26,9 +26,9 @@ from .eonode import EONode, linearly_connect_tasks
 from .eotask import EOTask
 from .eoworkflow import EOWorkflow, WorkflowResults
 from .eoworkflow_tasks import OutputTask
-from .utils.common import constant_pad, deep_eq
-from .utils.fs import get_filesystem, load_s3_filesystem
+from .utils.common import deep_eq
+from .utils.fs import get_filesystem, load_s3_filesystem, pickle_fs, unpickle_fs
 from .utils.parallelize import execute_with_mp_lock, join_futures, join_futures_iter, parallelize
 from .utils.parsing import FeatureParser
 
-__version__ = "1.1.0"
+__version__ = "1.2.0"
