@@ -16,12 +16,14 @@ file in the root directory of this source tree.
 """
 import concurrent.futures
 import datetime as dt
+import inspect
 import logging
+import sys
 import threading
 import warnings
 from dataclasses import dataclass
-from logging import Filter, Handler, Logger
-from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple
+from logging import FileHandler, Filter, Handler, Logger
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 import fs
 from fs.base import FS
@@ -29,12 +31,25 @@ from fs.base import FS
 from .eonode import EONode
 from .eoworkflow import EOWorkflow, WorkflowResults
 from .exceptions import EORuntimeWarning
-from .utils.fs import get_base_filesystem_and_path, get_full_path
+from .utils.fs import get_base_filesystem_and_path, get_full_path, pickle_fs, unpickle_fs
 from .utils.logging import LogFileFilter
 from .utils.parallelize import _decide_processing_type, _ProcessingType, parallelize
 
+if sys.version_info < (3, 8):
+    from typing_extensions import Protocol
+else:
+    from typing import Protocol  # pylint: disable=ungrouped-imports
+
+
+class _HandlerWithFsFactoryType(Protocol):
+    """Type definition for a callable that accepts a path and a filesystem object"""
+
+    def __call__(self, path: str, filesystem: FS, **kwargs: Any) -> Handler:
+        ...
+
+
 # pylint: disable=invalid-name
-_HandlerFactoryType = Callable[[str], Handler]
+_HandlerFactoryType = Union[Callable[[str], Handler], _HandlerWithFsFactoryType]
 
 
 @dataclass(frozen=True)
@@ -44,6 +59,7 @@ class _ProcessingData:
 
     workflow: EOWorkflow
     workflow_kwargs: Dict[EONode, Dict[str, object]]
+    pickled_filesystem: bytes
     log_path: Optional[str]
     filter_logs_by_thread: bool
     logs_filter: Optional[Filter]
@@ -78,7 +94,7 @@ class EOExecutor:
         logs_folder: str = ".",
         filesystem: Optional[FS] = None,
         logs_filter: Optional[Filter] = None,
-        logs_handler_factory: _HandlerFactoryType = logging.FileHandler,
+        logs_handler_factory: _HandlerFactoryType = FileHandler,
     ):
         """
         :param workflow: A prepared instance of EOWorkflow class
@@ -93,8 +109,14 @@ class EOExecutor:
         :param filesystem: A filesystem object for saving logs and a report.
         :param logs_filter: An instance of a custom filter object that will filter certain logs from being written into
             logs. It works only if save_logs parameter is set to True.
-        :param logs_handler_factory: A callable class or function that takes logging path as its only input parameter
-            and creates an instance of logging handler object
+        :param logs_handler_factory: A callable class or function that initializes an instance of a logging `Handler`
+            object. Its signature should support one of the following options:
+
+            - A single parameter describing a full path to the log file.
+            - Parameters `path` and `filesystem` where path to the log file is relative to the given `filesystem`
+              object.
+
+            The 2nd option is chosen only if `filesystem` parameter exists in the signature.
         """
         self.workflow = workflow
         self.execution_kwargs = self._parse_and_validate_execution_kwargs(execution_kwargs)
@@ -164,13 +186,18 @@ class EOExecutor:
         if self.save_logs:
             self.filesystem.makedirs(self.report_folder, recreate=True)
 
-        log_paths = self.get_log_paths(full_path=True) if self.save_logs else [None] * len(self.execution_kwargs)
+        log_paths: Sequence[Optional[str]]
+        if self.save_logs:
+            log_paths = self.get_log_paths(full_path=False)
+        else:
+            log_paths = [None] * len(self.execution_kwargs)
 
         filter_logs_by_thread = not multiprocess and workers is not None and workers > 1
         processing_args = [
             _ProcessingData(
                 workflow=self.workflow,
                 workflow_kwargs=workflow_kwargs,
+                pickled_filesystem=pickle_fs(self.filesystem),
                 log_path=log_path,
                 filter_logs_by_thread=filter_logs_by_thread,
                 logs_filter=self.logs_filter,
@@ -204,6 +231,7 @@ class EOExecutor:
     def _try_add_logging(
         cls,
         log_path: Optional[str],
+        pickled_filesystem: bytes,
         filter_logs_by_thread: bool,
         logs_filter: Optional[Filter],
         logs_handler_factory: _HandlerFactoryType,
@@ -213,7 +241,9 @@ class EOExecutor:
             try:
                 logger = logging.getLogger()
                 logger.setLevel(logging.DEBUG)
-                handler = cls._build_log_handler(log_path, filter_logs_by_thread, logs_filter, logs_handler_factory)
+                handler = cls._build_log_handler(
+                    log_path, pickled_filesystem, filter_logs_by_thread, logs_filter, logs_handler_factory
+                )
                 logger.addHandler(handler)
                 return logger, handler
             except BaseException as exception:
@@ -235,7 +265,11 @@ class EOExecutor:
     def _execute_workflow(cls, data: _ProcessingData) -> WorkflowResults:
         """Handles a single execution of a workflow."""
         logger, handler = cls._try_add_logging(
-            data.log_path, data.filter_logs_by_thread, data.logs_filter, data.logs_handler_factory
+            data.log_path,
+            data.pickled_filesystem,
+            data.filter_logs_by_thread,
+            data.logs_filter,
+            data.logs_handler_factory,
         )
 
         results = data.workflow.execute(data.workflow_kwargs, raise_errors=False)
@@ -246,19 +280,27 @@ class EOExecutor:
     @staticmethod
     def _build_log_handler(
         log_path: str,
+        pickled_filesystem: bytes,
         filter_logs_by_thread: bool,
         logs_filter: Optional[Filter],
         logs_handler_factory: _HandlerFactoryType,
     ) -> Handler:
         """Provides object which handles logs."""
-        handler = logs_handler_factory(log_path)
+        filesystem = unpickle_fs(pickled_filesystem)
+
+        factory_signature = inspect.signature(logs_handler_factory)
+        if "filesystem" in factory_signature.parameters:
+            handler = logs_handler_factory(log_path, filesystem=filesystem)  # type: ignore[call-arg]
+        else:
+            full_path = get_full_path(filesystem, log_path)
+            handler = logs_handler_factory(full_path)  # type: ignore[call-arg]
 
         if not handler.formatter:
             formatter = logging.Formatter("%(asctime)s %(name)-12s %(levelname)-8s %(message)s")
             handler.setFormatter(formatter)
 
         if filter_logs_by_thread:
-            handler.addFilter(LogFileFilter(threading.currentThread().getName()))
+            handler.addFilter(LogFileFilter(threading.current_thread().name))
 
         if logs_filter:
             handler.addFilter(logs_filter)
@@ -270,7 +312,7 @@ class EOExecutor:
         """Provides a type of processing according to given parameters."""
         return _decide_processing_type(workers=workers, multiprocess=multiprocess)
 
-    def _prepare_general_stats(self, workers: int, processing_type: _ProcessingType) -> Dict[str, object]:
+    def _prepare_general_stats(self, workers: Optional[int], processing_type: _ProcessingType) -> Dict[str, object]:
         """Prepares a dictionary with a general statistics about executions."""
         failed_count = sum(results.workflow_failed() for results in self.execution_results)
         return {
